// <PEdit2.cpp> ver.0.5
// Created By T.Kanda 1997.07.18
//

#include "PEdit.h"
#include "PEditWindow.h"
#include "PEditTextView.h"
#include "JumpDialog.h"
#include "FindDialog.h"

int        FontSizeMax = 9;
const char *FontSize[9] = { "9", "10", "12", "14", "18", "24", "36", "48", "72" };

PEWindow::PEWindow(BRect frame)
:BWindow(frame, "PEdit", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
    mFilePanelOpen = new BFilePanel(B_OPEN_PANEL);
    mFilePanelSave = new BFilePanel(B_SAVE_PANEL);
    for (int i=0; i<PAGE_MAX; i++) {
        mEditMode[i] = 0;
        mSearchPoint[i] = 0;
    }
    mSizePoint   = DEFAULT_SIZE_POINT;
    mFontSize    = atoi(FontSize[mSizePoint]);

	InitWindow();

    mUntitledNum = 0;
    mMaxPage = 0;

    mDir_Open.SetTo(DEFAULT_DIRECTORY);
    mDir_Save.SetTo(DEFAULT_DIRECTORY);
}

PEWindow::~PEWindow()
{
    delete mFilePanelOpen;
    delete mFilePanelSave;
}

void PEWindow::FrameResized(float /* width */, float /* height */)
{
//	ReCalcScrollBars(0);
}

void PEWindow::InitWindow()
{
	// setup menu bar
	BRect rect = Bounds();
	rect.bottom = rect.top + MENU_BAR_HEIGHT - 1;
	BMenuBar *mainmenu = new BMenuBar(rect, "",
            					       B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP,
							           B_ITEMS_IN_ROW, true );

	BMenu *file_menu = new BMenu("File");
	file_menu->AddItem(mNew = new BMenuItem("New", new BMessage(MENU_NEW), 'N'));
	file_menu->AddItem(mNewWindow = new BMenuItem("New Window", new BMessage(MENU_NEW_WINDOW)));
	file_menu->AddItem(mOpen = new BMenuItem("Open...", new BMessage(MENU_OPEN), 'O'));
	file_menu->AddItem(mOpenWithNewWindow = new BMenuItem("Open With New Window", new BMessage(MENU_OPEN_WITH_NEW_WINDOW)));
    file_menu->AddSeparatorItem();
	file_menu->AddItem(mSave = new BMenuItem("Save", new BMessage(MENU_SAVE), 'S'));
	file_menu->AddItem(mSaveAs = new BMenuItem("Save As...", new BMessage(MENU_SAVE_AS)));
	file_menu->AddItem(mRevert = new BMenuItem("Revert", new BMessage(MENU_REVERT)));
    file_menu->AddSeparatorItem();
	file_menu->AddItem(mPageSetup = new BMenuItem("Page Setup", new BMessage(MENU_PAGE_SETUP)));
	file_menu->AddItem(mPrint = new BMenuItem("Print", new BMessage(MENU_PRINT)));
    file_menu->AddSeparatorItem();
//	file_menu->AddItem(fCloseWindow = new BMenuItem("Close Window", new BMessage(MENU_CLOSE_WINDOW)));
	file_menu->AddItem(mCloseWindow = new BMenuItem("Close Window", new BMessage(B_QUIT_REQUESTED)));
//	file_menu->AddItem(new BMenuItem("Close", new BMessage(B_QUIT_REQUESTED), 'W'));
	file_menu->AddItem(mClose = new BMenuItem("Close", new BMessage(MENU_CLOSE), 'W'));

	BMenu *edit_menu = new BMenu("Edit");
	edit_menu->AddItem(mUndo = new BMenuItem("Undo", new BMessage(MENU_UNDO), 'Z'));
    edit_menu->AddSeparatorItem();
	edit_menu->AddItem(mCut = new BMenuItem("Cut", new BMessage(MENU_CUT), 'X'));
	edit_menu->AddItem(mCopy = new BMenuItem("Copy", new BMessage(MENU_COPY), 'C'));
	edit_menu->AddItem(mPaste = new BMenuItem("Paste", new BMessage(MENU_PASTE), 'V'));
	edit_menu->AddItem(mClear = new BMenuItem("Clear", new BMessage(MENU_CLEAR)));
    edit_menu->AddSeparatorItem();
	edit_menu->AddItem(mSelectAll = new BMenuItem("Select All", new BMessage(MENU_SELECT_ALL), 'A'));
    edit_menu->AddSeparatorItem();
	edit_menu->AddItem(mFind = new BMenuItem("Find", new BMessage(MENU_FIND), 'F'));
	edit_menu->AddItem(mFindAgain = new BMenuItem("Find Again", new BMessage(MENU_FIND_AGAIN), 'G'));
    edit_menu->AddSeparatorItem();
	edit_menu->AddItem(mJump = new BMenuItem("Jump", new BMessage(MENU_JUMP), 'J'));

    GetFontInfo();

	BMenu *font_menu = new BMenu("Font");
    BMenu *menu = new BMenu("Size");
    menu->SetRadioMode(true);
    for (int i=0; i<FontSizeMax; i++) {
        menu->AddItem(mSize[i] = new BMenuItem(FontSize[i], new BMessage(MENU_SIZE)));
	}
	font_menu->AddItem(menu);
    font_menu->AddSeparatorItem();
    for (int i=0; i<mNumFamilies; i++) {
        mFontName[i] = new BMenu(mFontInfo[i].FontName);
        mFontName[i]->SetRadioMode(true);
    	for (int j=0; j<mFontInfo[i].NumStyles; j++) {
            mFontName[i]->AddItem(mStyle[i][j] = new BMenuItem(mFontInfo[i].FontStyle[j], new BMessage(MENU_FONT + i)));
    	}
    	font_menu->AddItem(mFontSubmenu[i] = new BMenuItem(mFontName[i]));
    }

	BMenu *info_menu = new BMenu("Info");
	info_menu->AddItem(mAbout = new BMenuItem("About PEdit", new BMessage(MENU_ABOUT)));
	info_menu->AddItem(mHelp = new BMenuItem("Help...", new BMessage(MENU_HELP)));

	mainmenu->AddItem(file_menu);
	mainmenu->AddItem(edit_menu);
	mainmenu->AddItem(font_menu);
	mainmenu->AddItem(info_menu);
	
	Lock();
	AddChild(mainmenu);
	Unlock();

    mNewWindow->SetEnabled(false); // now debug
    mOpenWithNewWindow->SetEnabled(false); // now debug
    mSave->SetEnabled(false);
    mSaveAs->SetEnabled(false);
    mRevert->SetEnabled(false);
    mCloseWindow->SetEnabled(false); // now debug
    mClose->SetEnabled(false);
    mCut->SetEnabled(false);
    mCopy->SetEnabled(false);
    mPaste->SetEnabled(false);
    mSelectAll->SetEnabled(false);
    mFind->SetEnabled(false);
    mFindAgain->SetEnabled(false);
    mJump->SetEnabled(false);

    mSize[mSizePoint]->SetMarked(true);
    bool SetFlag = false;
    for (int i=0; i<mNumFamilies; i++) {
        if (!strcmp(mFontInfo[i].FontName, DEFAULT_FONT_FAMILY)) {
            mFamilyPoint = i;
            mFontSubmenu[i]->SetMarked(true);
            for (int j=0; j<mFontInfo[i].NumStyles; j++) {
                if (!strcmp(mFontInfo[i].FontStyle[j], DEFAULT_FONT_STYLE)) {
                    mStylePoint = j;
                    mStyle[i][j]->SetMarked(true);
                    SetFlag = true;
                    break;
                }
            }
        }
        if (SetFlag) break;
    }


	// setup tool bar
	rect.top = mainmenu->Bounds().bottom + 1;
	rect.bottom = rect.top + TOOL_BAR_HEIGHT - 1;
	BView *toolBar = new BView(rect, "", B_FOLLOW_NONE, B_WILL_DRAW);
	toolBar->SetViewColor(kControl);

    CToolButton *btn;
	float x_pos = 0.0;
	
    rect.Set(0, 0, TOOL_BUTTON_WIDTH-1, TOOL_BUTTON_HEIGHT-1);
	rect.OffsetTo(x_pos, 0);
	btn = new CToolButton(rect, "", get_icon_image(NEW_ICON), get_icon_image(NEW_MASK_ICON), "New", B_ALIGN_LEFT, new BMessage(BUTTON_NEW));
    toolBar->AddChild(btn);

    x_pos += TOOL_BUTTON_WIDTH;
	rect.OffsetTo(x_pos, 0);
	btn = new CToolButton(rect, "", get_icon_image(OPEN_ICON), get_icon_image(OPEN_MASK_ICON), "Open...", B_ALIGN_LEFT, new BMessage(BUTTON_OPEN));
    toolBar->AddChild(btn);

    x_pos += TOOL_BUTTON_WIDTH;
	rect.OffsetTo(x_pos, 0);
	btn = new CToolButton(rect, "", get_icon_image(SAVE_ICON), get_icon_image(SAVE_MASK_ICON), "Save", B_ALIGN_LEFT, new BMessage(BUTTON_SAVE));
    toolBar->AddChild(btn);

    x_pos += TOOL_BUTTON_WIDTH;
	rect.OffsetTo(x_pos, 0);
	btn = new CToolButton(rect, "", get_icon_image(PRINT_ICON), get_icon_image(PRINT_MASK_ICON), "Print", B_ALIGN_LEFT, new BMessage(BUTTON_PRINT));
    toolBar->AddChild(btn);

    x_pos += TOOL_BUTTON_WIDTH;
	rect.OffsetTo(x_pos, 0);
	btn = new CToolButton(rect, "", get_icon_image(UNDO_ICON), get_icon_image(UNDO_MASK_ICON), "Undo", B_ALIGN_LEFT, new BMessage(BUTTON_UNDO));
    toolBar->AddChild(btn);

    x_pos += TOOL_BUTTON_WIDTH;
	rect.OffsetTo(x_pos, 0);
	btn = new CToolButton(rect, "", get_icon_image(CUT_ICON), get_icon_image(CUT_MASK_ICON), "Cut", B_ALIGN_LEFT, new BMessage(BUTTON_CUT));
    toolBar->AddChild(btn);

    x_pos += TOOL_BUTTON_WIDTH;
	rect.OffsetTo(x_pos, 0);
	btn = new CToolButton(rect, "", get_icon_image(COPY_ICON), get_icon_image(COPY_MASK_ICON), "Copy", B_ALIGN_LEFT, new BMessage(BUTTON_COPY));
    toolBar->AddChild(btn);

    x_pos += TOOL_BUTTON_WIDTH;
	rect.OffsetTo(x_pos, 0);
	btn = new CToolButton(rect, "", get_icon_image(PASTE_ICON), get_icon_image(PASTE_MASK_ICON), "Paste", B_ALIGN_LEFT, new BMessage(BUTTON_PASTE));
    toolBar->AddChild(btn);

    x_pos += TOOL_BUTTON_WIDTH;
	rect.OffsetTo(x_pos, 0);
	btn = new CToolButton(rect, "", get_icon_image(FIND_ICON), get_icon_image(FIND_MASK_ICON), "Find", B_ALIGN_LEFT, new BMessage(BUTTON_FIND));
    toolBar->AddChild(btn);
    
	Lock();
	AddChild(toolBar);
	Unlock();

	// setup tags view
	mPageRect = Bounds();
	mPageRect.top = toolBar->Frame().bottom + 1;
	mTagsView = new CTagsView(mPageRect, "");
	Lock();
	AddChild(mTagsView);
	Unlock();
}

void PEWindow::ReadFile(entry_ref *ref)
{
    if (mMaxPage < PAGE_MAX) {
        mMaxPage++;
        mEditMode[mMaxPage-1] = 2; // Open Type
  	    mTagsView->AddPage("", "    ");
        Lock();
        CreateView(mMaxPage - 1);
        Unlock();
    } else {
        (new BAlert("", "Sorry. Page is full.", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();
    }
    long  index = mTagsView->CurrentPage();
    char  f_name[B_FILE_NAME_LENGTH];
    off_t size;

    if (mFile.SetTo(ref, B_READ_WRITE) == B_OK) {
        mDir_Open.SetTo(ref);
        mEntry[index].SetTo(ref);
        mEntry[index].GetName(f_name);
    } else {
        (new BAlert("", "Open Failed.", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT))->Go();
        return;
    }

    mFile.GetSize(&size);
    mBuff[index] = new char[size+1];
    long readByte = mFile.Read(mBuff[index], size);
	Lock();
    mTagsView->SetPageLabel(index, f_name);
    mScrollTextView[index]->SetText(mBuff[index], readByte);
	Unlock();
//    delete mBuff[index];
    strcpy(mLabel[index], f_name);
}

void PEWindow::WriteFile(entry_ref *ref, const char *f_name)
{
    long index = mTagsView->CurrentPage();

    mDir_Save.SetTo(ref);
    if (mDir_Save.CreateFile(f_name, &mFile, false) == B_NO_ERROR ) {
        mEntry[index].SetTo(&mDir_Save, f_name);
        mFile.SetTo(&mEntry[index], B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);
        mFile.Write(mScrollTextView[index]->Text(),
                           mScrollTextView[index]->TextLength());
        mEditMode[index] = 2; // Open Type
        Lock();
        mTagsView->SetPageLabel(index, (char *)f_name);
        Unlock();
        UpdateMenu();
        (new BAlert("", "Save Done.", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();
    }
}

void PEWindow::UpdateMenu()
{
    long index = mTagsView->CurrentPage();

    switch (mEditMode[index]) {
    case 1: // New
        mSave->SetEnabled(false);
        mSaveAs->SetEnabled(false);
        mRevert->SetEnabled(false);
        mClose->SetEnabled(true);
        mCut->SetEnabled(true);
        mCopy->SetEnabled(true);
        mPaste->SetEnabled(true);
        mSelectAll->SetEnabled(true);
        mFind->SetEnabled(true);
        mFindAgain->SetEnabled(true);
        mJump->SetEnabled(true);
        break;
    case 2: // Open
        mSave->SetEnabled(true);
        mSaveAs->SetEnabled(true);
        mRevert->SetEnabled(true);
        mClose->SetEnabled(true);
        mCut->SetEnabled(true);
        mCopy->SetEnabled(true);
        mPaste->SetEnabled(true);
        mSelectAll->SetEnabled(true);
        mFind->SetEnabled(true);
        mFindAgain->SetEnabled(true);
        mJump->SetEnabled(true);
        break;
    }
}

void PEWindow::GetFontInfo()
{
    mNumFamilies = count_font_families();
    
    for (int i=0; i<mNumFamilies; i++) {
        font_family family;
        uint32 flags;
        if (get_font_family(i, &family, &flags) == B_OK) {
            strcpy(mFontInfo[i].FontName, family);
            int32 numStyles = count_font_styles(family);
            if (i <32) mFontInfo[i].NumStyles = numStyles;
            for (int j=0; j<numStyles; j++) {
                font_style style;
                if (get_font_style(family, j, &style, &flags) == B_OK) {
                    if (j < 8) strcpy(mFontInfo[i].FontStyle[j], style);
                }
            }
        }
    }
}

void PEWindow::SetLineNo(int32 LineNo)
{
    int32 index = mTagsView->CurrentPage();
    Lock();
   	if (mScrollTextView[index]->JumpToLine(LineNo) == false) {
        (new BAlert("", "Cannot Jump!", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();
   	}
    Unlock();
}

void PEWindow::FindWord(const char *terget)
{
    int32 index = mTagsView->CurrentPage();
    mSearchPoint[index] = 0;
    strcpy(mTerget, terget);
    Lock();
   	if (mScrollTextView[index]->FindWord(&mSearchPoint[index], mTerget) == false) {
        (new BAlert("", "Not Found!", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();
   	}
    Unlock();
}

void PEWindow::MessageReceived(BMessage *message)
{
    long     index;
    char     label[32];
    off_t    size;
    BMessage *msg;
    BRect     rect;
    
	if (message) {
        switch (message->what) {
        case MENU_NEW:
        case BUTTON_NEW:
            if (mMaxPage < PAGE_MAX) {
                mUntitledNum++;
                mMaxPage++;
                mEditMode[mMaxPage-1] = 1; // New Type
                sprintf(label, "Untitled%ld", mUntitledNum);
        	    mTagsView->AddPage("", label);
                Lock();
                CreateView(mMaxPage - 1);
                Unlock();
                strcpy(mLabel[mMaxPage - 1], label);
	        } else {
                (new BAlert("", "Sorry. Page is full.", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();
	        }
	        break;
        case MENU_OPEN:
        case BUTTON_OPEN:
            mFilePanelOpen->SetPanelDirectory(&mDir_Open);
            mFilePanelOpen->Show();
            break;
        case MENU_NEW_WINDOW:
            msg = new BMessage(MSG_NEW_WINDOW);
            be_app->PostMessage(msg);
            break;
        case MENU_SAVE:
        case BUTTON_SAVE:
            if (!mMaxPage) return;
            index = mTagsView->CurrentPage();

            switch (mEditMode[index]) {
            case 1: // New
                mFilePanelSave->SetPanelDirectory(&mDir_Save);
                mFilePanelSave->SetSaveText(mLabel[index]);
                mFilePanelSave->Show();
                break;
            case 2: // Open
                mFile.SetTo(&mEntry[index], B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);
                mFile.Write(mScrollTextView[index]->Text(),
                            mScrollTextView[index]->TextLength());
                (new BAlert("", "Save Done.", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();
            }
            break;
        case MENU_SAVE_AS:
            index = mTagsView->CurrentPage();

            mFilePanelSave->SetPanelDirectory(&mDir_Save);
            mFilePanelSave->SetSaveText(mLabel[index]);
            mFilePanelSave->Show();
            break;
        case MENU_REVERT:
            if (!mMaxPage) return;
            index = mTagsView->CurrentPage();
            mFile.SetTo(&mEntry[index], B_READ_WRITE);
            mFile.GetSize(&size);
            mBuff[index] = new char[size+1];
            long readByte = mFile.Read(mBuff[index], size);
        	Lock();
            mScrollTextView[index]->SetText(mBuff[index], readByte);
        	Unlock();
//    delete mBuff[index];
            break;
        case MENU_PAGE_SETUP:
            (new BAlert("", "'Page Setup' not yet....", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();
            break;
        case MENU_PRINT:
        case BUTTON_PRINT:
            if (!mMaxPage) return;
            (new BAlert("", "'Print' not yet....", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();
	        break;
//        case MENU_CLOSE_WINDOW:
//            msg = new BMessage(MSG_CLOSE_WINDOW);
//            be_app->PostMessage(msg);
//            break;
        case MENU_CLOSE:
            index = mTagsView->CurrentPage();
//            delete mBuff[index];
            mMaxPage--;
            for (int i=index; i<mMaxPage; i++) {
                mPage[i]            = mPage[i+1];
                mScrollTextView[i] = mScrollTextView[i+1];
                mEntry[i]          = mEntry[i+1];
                mBuff[i]            = mBuff[i+1];
                mEditMode[i]       = mEditMode[i+1];
                mSearchPoint[i]    = mSearchPoint[i+1];
                strcpy(mLabel[i], mLabel[i+1]);
            }
       	    mTagsView->DeletePage(index);
            if (mMaxPage == 0) {
                be_app->SetCursor(B_HAND_CURSOR);
                mClose->SetEnabled(false);
                mCut->SetEnabled(false);
                mCopy->SetEnabled(false);
                mPaste->SetEnabled(false);
                mSelectAll->SetEnabled(false);
                mFind->SetEnabled(false);
                mFindAgain->SetEnabled(false);
                mJump->SetEnabled(false);
            }
            break;
        case BUTTON_UNDO:
            if (!mMaxPage) return;
            (new BAlert("", "'Undo' not yet....", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();
	        break;
        case MENU_CUT:
        case BUTTON_CUT:
            if (!mMaxPage) return;
            index = mTagsView->CurrentPage();
            if (mEditMode[index] != 0) {
                mScrollTextView[index]->Cut(be_clipboard);
	        }
	        break;
        case MENU_COPY:
        case BUTTON_COPY:
            if (!mMaxPage) return;
            index = mTagsView->CurrentPage();
            if (mEditMode[index] != 0) {
                mScrollTextView[index]->Copy(be_clipboard);
	        }
	        break;
        case MENU_PASTE:
        case BUTTON_PASTE:
            if (!mMaxPage) return;
            index = mTagsView->CurrentPage();
            if (mEditMode[index] != 0) {
                mScrollTextView[index]->Paste(be_clipboard);
            }
	        break;
	    case MENU_SELECT_ALL:
            index = mTagsView->CurrentPage();
            if (mEditMode[index] != 0) {
                mScrollTextView[index]->SelectAll();
            }
            break;
        case MENU_FIND:
        case BUTTON_FIND:
            if (!mMaxPage) return;
            rect.Set(0, 0, 460, 33);
 //           BRect pos = Frame();
            rect.OffsetBy(40, 200);
            mFindDialog = new PEFindDialog(this, rect);
	
         	mFindDialog->Show();
	        break;
        case MENU_FIND_AGAIN:
            index = mTagsView->CurrentPage();
            mScrollTextView[index]->FindWord(&mSearchPoint[index], mTerget);
	        break;
        case MENU_JUMP:
            rect.Set(0, 0, 260, 33);
            rect.OffsetBy(40, 200);
            mJumpDialog = new PEJumpDialog(this, rect);
	
         	mJumpDialog->Show();
	        break;
        case MENU_SIZE:
            for (int i=0; i<9; i++) {
                if (mSize[i]->IsMarked()) {
                    mFontSize = atoi(FontSize[i]);
                    break;
                }
            }
            break;
        case MENU_FONT:
            bool SetFlag = false;
            for (int i=0; i<mNumFamilies; i++) {
                for (int j=0; j<mFontInfo[i].NumStyles; j++) {
                    if (mStyle[i][j]->IsMarked()) {
                        SetFlag = true;
                        mFamilyPoint = i;
                        mStylePoint = j;
                        break;
                    }
                }
                if (SetFlag) break;
            }
            break;
        case MENU_ABOUT:
            (new BAlert("", "PEdit2 ver.0.5", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();
            break;
        case MENU_HELP:
            (new BAlert("", "'Help' not yet....", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();
            break;
        case C_SCROLL_TEXT_VIEW_ATTACHED:
        	be_app->SetCursor(B_HAND_CURSOR);
            UpdateMenu();
            break;
        default:
            if (MENU_FONT <= message->what && message->what < MENU_FONT + 32) {
                mFamilyPoint = message->what - MENU_FONT;
                mStylePoint = message->FindInt32("index");
                for (int i=0; i<mNumFamilies; i++) {
                    mFontSubmenu[i]->SetMarked(false);
                    for (int j=0; j<mFontInfo[i].NumStyles; j++) {
                        mStyle[i][j]->SetMarked(false);
                    }
                }
                mFontSubmenu[mFamilyPoint]->SetMarked(true);
                mStyle[mFamilyPoint][mStylePoint]->SetMarked(true);
            }
        }
    }
}

bool PEWindow::QuitRequested()
{
//	be_app->PostMessage(B_QUIT_REQUESTED);
    BMessage *msg = new BMessage(MSG_CLOSE_WINDOW);
    be_app->PostMessage(msg);

	return(true);
}

void PEWindow::CreateView(long index)
{
    mTagsView->ChangePage(index);
    mPage[index] = mTagsView->PageAt(index);
	
	BRect rect(0, 6, mPage[index]->Bounds().Width() - 1, mPage[index]->Bounds().Height() - 1);
	rect.InsetBy(8, 8);

	mScrollTextView[index] = new PETextView(rect, "");
	mPage[index]->AddChild(mScrollTextView[index]);

////	mScrollTextView[index]->SetViewColor(kControl);

    BFont font;
    font.SetFamilyAndStyle(mFontInfo[mFamilyPoint].FontName,
                           mFontInfo[mFamilyPoint].FontStyle[mStylePoint]);
    font.SetSize(mFontSize);
    mScrollTextView[index]->SetFont(&font, 0);
}

